/*
 * Copyright (c) 2015 - 2016, Freescale Semiconductor, Inc.
 * Copyright 2016 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdio.h"
#include "string.h"
#include "usb_host_config.h"
#include "host_msd_fatfs.h"
#include "usb_misc.h"
#include "diskio.h"
#include "fsl_device_registers.h"
#include "fsl_debug_console.h"

#include "BANSHI_ff.h"

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/**
 * @def STATIC
 * @brief This defines variable data type static
 */
#define STATIC static

/**
 * @def USB_FILENAME_LENGTH
 * @brief This defines length for the file name
 */
#define USB_FILENAME_LENGTH                 (11u)

/**
 * @def USB_NOT_MOUNTED
 * @brief This defines USB is not mounted on any drive path
 */
#define USB_NOT_MOUNTED                     (0xFF)

/*******************************************************************************
 * Prototypes
 ******************************************************************************/
/*!
 * @brief host msd control transfer callback.
 *
 * This function is used as callback function for control transfer .
 *
 * @param param      the host msd fatfs instance pointer.
 * @param data       data buffer pointer.
 * @param dataLength data length.
 * @status           transfer result status.
 */
void USB_HostMsdControlCallback(void *param, uint8_t *data, uint32_t dataLength, usb_status_t status);

/*******************************************************************************
 * Variables
 ******************************************************************************/
/*! @brief msd class handle array for fatfs */
extern usb_host_class_handle g_UsbFatfsClassHandle;
extern usb_host_class_handle g_UsbFatfsClassHandle_1;

/* global msd fatfs instance */
usb_host_msd_fatfs_instance_t g_MsdFatfsInstance;
usb_host_msd_fatfs_instance_t g_MsdFatfsInstance_1;

/* fatfs instance */
FATFS fatfs[2];

/* control transfer on-going state. It should set to 1 when start control transfer, it is set to 0 in the callback */
volatile uint8_t controlIng;

/* control transfer callback status */
volatile usb_status_t controlStatus;

/* indicates the drive number on which external media is mounted */
STATIC uint8_t ucExternalMediaPath = USB_NOT_MOUNTED;

/* indicates the drive number on which internal media is mounted */
STATIC uint8_t ucInternalMediaPath = USB_NOT_MOUNTED;

/* flag to indicate internal media is mounted or not */
STATIC bool bUSB0MountStatus = false;

/* array for storing drive path */
uint8_t driverNumberBuffer[3];
/*******************************************************************************
 * Code
 ******************************************************************************/
void USB_HostMsdControlCallback(void *param, uint8_t *data, uint32_t dataLength, usb_status_t status)
{
    usb_host_msd_fatfs_instance_t *msdFatfsInstance = (usb_host_msd_fatfs_instance_t *)param;

    if (msdFatfsInstance->runWaitState == kRunWaitSetInterface) /* set interface finish */
    {
        msdFatfsInstance->runWaitState = kRunIdle;
        msdFatfsInstance->runState = kRunMassStorage;
    }
    controlIng = 0;
    controlStatus = status;
}

#ifdef BOOTLOADER_USB
bool GetMediaStatus(etMediaSource_t etMedia)
{
    switch(etMedia)
    {
        case kMediaSourceInternal:
        {
            if(USB_NOT_MOUNTED == ucInternalMediaPath)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        case kMediaSourceExternal:
        {
            if(USB_NOT_MOUNTED == ucExternalMediaPath)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        default:
        {
            return false;
        }
    }/* end of switch */
}
#endif

/**
 * \brief returns the media path
 * \param[in]	etMedia
 * \return      void
 */
uint8_t GetMediaPath(etMediaSource_t etMedia)
{
    switch(etMedia)
    {
        case kMediaSourceInternal:
        {
            return ucInternalMediaPath;
        }

        case kMediaSourceExternal:
        {
            return ucExternalMediaPath;
        }

        default:
        {
            return USB_NOT_MOUNTED;
        }
    }
}

/**
 * \brief       Function used to validate the memory availability.
 * \param[in]   me          class pointer
 * \return      void
 */
void FATFS_Init(uint8_t ucnUSBDrive)
{
    FRESULT returnCode;
    TCHAR label[USB_FILENAME_LENGTH] = {0};
    uint32_t unSerNum = 0;

#ifdef BOOTLOADER_USB
	uint32_t resultSize;
    /* time delay (~100ms) */
    for (resultSize = 0; resultSize < 400000; ++resultSize){
        __ASM("nop");
    }
#endif

    /* select drive for mounting */
    if(ucnUSBDrive)
    {
        snprintf((char *)&driverNumberBuffer[0],
                sizeof(driverNumberBuffer),
                "%c:", 1 + '0');
    }
    else
    {
        snprintf((char *)&driverNumberBuffer[0],
                sizeof(driverNumberBuffer),
                "%c:", 0 + '0');
    }

    /* Change Current Directory or Current Drive */
    if(ucnUSBDrive)
    {
        returnCode = BANSHI_f_chdrive((char const *)&driverNumberBuffer[0]);
        if (returnCode)	{
            usb_echo("error\r\n");
            return;
        }
    }

    /* Initialize logical driver */
    returnCode = BANSHI_f_mount(&fatfs[ucnUSBDrive], (char const *)&driverNumberBuffer[0], 0);
    if (returnCode)	{
        usb_echo("error\r\n");
        return;
    }

    /* Send some SCSI requests first */
    returnCode = (FRESULT)disk_initialize(ucnUSBDrive);
    if (returnCode)	{
        usb_echo("error\r\n");
        return;
    }

    /* get the lable for mounted mass storage device */
    returnCode = BANSHI_f_getlabel((const char *)&driverNumberBuffer[0], label, &unSerNum);
    if (returnCode)	{
        usb_echo("error\r\n");
        return;
    }

    /* compare label to decide external media storage or
     * internal media storage */
    if(0 == strcmp(label, "USB_1"))
    {
        /* save path for drive */
        ucExternalMediaPath = ucnUSBDrive;
    }
    else
    {
        if(false == bUSB0MountStatus)
        {
            /* save path for drive */
            ucInternalMediaPath = ucnUSBDrive;
            /* label for media */
            memset(label,0,sizeof(label));
            strcpy(label, "USB_0");
            /* label media as a USB_0 */
            BANSHI_f_setlabel(label);

            /* status USB0 mounted */
            bUSB0MountStatus = true;
        }
    }
}

void USB_HostMsdTask(void *arg)
{
    usb_status_t status;
    usb_host_msd_fatfs_instance_t *msdFatfsInstance = (usb_host_msd_fatfs_instance_t *)arg;

    if (msdFatfsInstance->deviceState != msdFatfsInstance->prevDeviceState)
    {
        msdFatfsInstance->prevDeviceState = msdFatfsInstance->deviceState;
        switch (msdFatfsInstance->deviceState)
        {
            case kStatus_DEV_Idle:
                break;

            case kStatus_DEV_Attached: /* deivce is attached and numeration is done */
                status = USB_HostMsdInit(msdFatfsInstance->deviceHandle,
                                         &msdFatfsInstance->classHandle); /* msd class initialization */
                if(&g_MsdFatfsInstance == arg)
                {
                    g_UsbFatfsClassHandle = msdFatfsInstance->classHandle;
                }
                else if(&g_MsdFatfsInstance_1 == arg)
                {
                    g_UsbFatfsClassHandle_1 = msdFatfsInstance->classHandle;
                }

                if (status != kStatus_USB_Success)
                {
                    usb_echo("usb host msd init fail\r\n");
                    return;
                }
                msdFatfsInstance->runState = kRunSetInterface;
                break;

            case kStatus_DEV_Detached: /* device is detached */
                msdFatfsInstance->deviceState = kStatus_DEV_Idle;
                msdFatfsInstance->runState = kRunIdle;
                USB_HostMsdDeinit(msdFatfsInstance->deviceHandle,
                                  msdFatfsInstance->classHandle); /* msd class de-initialization */
                msdFatfsInstance->classHandle = NULL;

                if(&g_MsdFatfsInstance == arg)
                {
                    snprintf((char *)&driverNumberBuffer[0],
                                    sizeof(driverNumberBuffer),
                                    "%c:", 0 + '0');
                    BANSHI_f_mount(NULL, (char const *)&driverNumberBuffer[0], 0);
                    bUSB0MountStatus = false;
                    ucInternalMediaPath = USB_NOT_MOUNTED;
                }
                else if(&g_MsdFatfsInstance_1 == arg)
                {
                    snprintf((char *)&driverNumberBuffer[0],
                                    sizeof(driverNumberBuffer),
                                    "%c:", 1 + '0');
                    BANSHI_f_mount(NULL, (char const *)&driverNumberBuffer[0], 0);
                    ucExternalMediaPath = USB_NOT_MOUNTED;
                }

                usb_echo("mass storage device detached\r\n");
                break;

            default:
                break;
        }
    }

    /* run state */
    switch (msdFatfsInstance->runState)
    {
        case kRunIdle:
            break;

        case kRunSetInterface: /* set msd interface */
            msdFatfsInstance->runState = kRunIdle;
            msdFatfsInstance->runWaitState = kRunWaitSetInterface;
            status = USB_HostMsdSetInterface(msdFatfsInstance->classHandle, msdFatfsInstance->interfaceHandle, 0,
                                             USB_HostMsdControlCallback, msdFatfsInstance);
            if (status != kStatus_USB_Success)
            {
                usb_echo("set interface fail\r\n");
            }
            break;

        case kRunMassStorageTest: /* set interface succeed */
            msdFatfsInstance->runState = kRunIdle;
            break;

        case kRunMassStorage:
        {
            if(&g_MsdFatfsInstance == arg)
            {
                FATFS_Init(0);
            }
            else if(&g_MsdFatfsInstance_1 == arg)
            {
                FATFS_Init(1);
            }
            msdFatfsInstance->runState = kRunIdle;
            break;
        }

        default:
            break;
    }
}

usb_status_t USB_HostMsdEvent(usb_device_handle deviceHandle,
                              usb_host_configuration_handle configurationHandle,
                              uint32_t eventCode)
{
    usb_status_t status = kStatus_USB_Success;
    usb_host_configuration_t *configuration;
    uint8_t interfaceIndex;
    usb_host_interface_t *interface;
    uint32_t infoValue;
    uint8_t id;

    switch (eventCode)
    {
        case kUSB_HostEventAttach:
            /* judge whether is configurationHandle supported */
            configuration = (usb_host_configuration_t *)configurationHandle;
            for (interfaceIndex = 0; interfaceIndex < configuration->interfaceCount; ++interfaceIndex)
            {
                interface = &configuration->interfaceList[interfaceIndex];
                id = interface->interfaceDesc->bInterfaceClass;
                if (id != USB_HOST_MSD_CLASS_CODE)
                {
                    continue;
                }
                id = interface->interfaceDesc->bInterfaceSubClass;
                if ((id != USB_HOST_MSD_SUBCLASS_CODE_UFI) && (id != USB_HOST_MSD_SUBCLASS_CODE_SCSI))
                {
                    continue;
                }
                id = interface->interfaceDesc->bInterfaceProtocol;
                if (id != USB_HOST_MSD_PROTOCOL_BULK)
                {
                    continue;
                }
                else
                {
                    if (g_MsdFatfsInstance.deviceState == kStatus_DEV_Idle)
                    {
                        /* the interface is supported by the application */
                        g_MsdFatfsInstance.deviceHandle = deviceHandle;
                        g_MsdFatfsInstance.interfaceHandle = interface;
                        g_MsdFatfsInstance.configHandle = configurationHandle;
                        return kStatus_USB_Success;
                    }
                    if (g_MsdFatfsInstance_1.deviceState == kStatus_DEV_Idle)
                    {
                        /* the interface is supported by the application */
                        g_MsdFatfsInstance_1.deviceHandle = deviceHandle;
                        g_MsdFatfsInstance_1.interfaceHandle = interface;
                        g_MsdFatfsInstance_1.configHandle = configurationHandle;
                        return kStatus_USB_Success;
                    }
                    else
                    {
                        continue;
                    }
                }
            }
            status = kStatus_USB_NotSupported;
            break;

        case kUSB_HostEventNotSupported:
            break;

        case kUSB_HostEventEnumerationDone:
            if (g_MsdFatfsInstance.configHandle == configurationHandle)
            {
                if ((g_MsdFatfsInstance.deviceHandle != NULL) && (g_MsdFatfsInstance.interfaceHandle != NULL))
                {
                    /* the device enumeration is done */
                    if (g_MsdFatfsInstance.deviceState == kStatus_DEV_Idle)
                    {
                        g_MsdFatfsInstance.deviceState = kStatus_DEV_Attached;

                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDevicePID, &infoValue);
                        usb_echo("mass storage device attached:pid=0x%x", infoValue);
                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDeviceVID, &infoValue);
                        usb_echo("vid=0x%x ", infoValue);
                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDeviceAddress, &infoValue);
                        usb_echo("address=%d\r\n", infoValue);
                    }
                    else
                    {
                        usb_echo("not idle msd instance\r\n");
                        status = kStatus_USB_Error;
                    }
                }
            }
            if (g_MsdFatfsInstance_1.configHandle == configurationHandle)
            {
                if ((g_MsdFatfsInstance_1.deviceHandle != NULL) && (g_MsdFatfsInstance_1.interfaceHandle != NULL))
                {
                    /* the device enumeration is done */
                    if (g_MsdFatfsInstance_1.deviceState == kStatus_DEV_Idle)
                    {
                        g_MsdFatfsInstance_1.deviceState = kStatus_DEV_Attached;

                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDevicePID, &infoValue);
                        usb_echo("mass storage device attached:pid=0x%x", infoValue);
                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDeviceVID, &infoValue);
                        usb_echo("vid=0x%x ", infoValue);
                        USB_HostHelperGetPeripheralInformation(deviceHandle, kUSB_HostGetDeviceAddress, &infoValue);
                        usb_echo("address=%d\r\n", infoValue);
                    }
                    else
                    {
                        usb_echo("not idle msd instance\r\n");
                        status = kStatus_USB_Error;
                    }
                }
            }
            break;

        case kUSB_HostEventDetach:
            if (g_MsdFatfsInstance.configHandle == configurationHandle)
            {
                /* the device is detached */
                g_UsbFatfsClassHandle = NULL;
                g_MsdFatfsInstance.configHandle = NULL;
                if (g_MsdFatfsInstance.deviceState != kStatus_DEV_Idle)
                {
                    g_MsdFatfsInstance.deviceState = kStatus_DEV_Detached;
                }
            }
            if (g_MsdFatfsInstance_1.configHandle == configurationHandle)
            {
                /* the device is detached */
                g_UsbFatfsClassHandle_1 = NULL;
                g_MsdFatfsInstance_1.configHandle = NULL;
                if (g_MsdFatfsInstance_1.deviceState != kStatus_DEV_Idle)
                {
                    g_MsdFatfsInstance_1.deviceState = kStatus_DEV_Detached;
                }
            }
            break;

        default:
            break;
    }
    return status;
}
